Package: utf8
Title: Unicode Text Processing
Version: 1.2.6
Authors@R: 
    c(person(given = c("Patrick", "O."),
             family = "Perry",
             role = c("aut", "cph")),
      person(given = "Kirill",
             family = "M\u00fcller",
             role = "cre",
             email = "kirill@cynkra.com",
             comment = c(ORCID = "0000-0002-1416-3412")),
      person(given = "Unicode, Inc.",
             role = c("cph", "dtc"),
             comment = "Unicode Character Database"))
Description: Process and print 'UTF-8' encoded international
    text (Unicode). Input, validate, normalize, encode, format, and
    display.
License: Apache License (== 2.0) | file LICENSE
URL: https://krlmlr.github.io/utf8/, https://github.com/krlmlr/utf8
BugReports: https://github.com/krlmlr/utf8/issues
Depends: R (>= 2.10)
Suggests: cli, covr, knitr, rlang, rmarkdown, testthat (>= 3.0.0),
        withr
VignetteBuilder: knitr, rmarkdown
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.3.2.9000
NeedsCompilation: yes
Packaged: 2025-06-08 18:28:11 UTC; kirill
Author: Patrick O. Perry [aut, cph],
  Kirill Müller [cre] (ORCID: <https://orcid.org/0000-0002-1416-3412>),
  Unicode, Inc. [cph, dtc] (Unicode Character Database)
Maintainer: Kirill Müller <kirill@cynkra.com>
Repository: CRAN
Date/Publication: 2025-06-08 19:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 08:46:38 UTC; unix
Archs: utf8.so.dSYM
