# Generated by roxygen2: do not edit by hand

export(as_utf8)
export(output_ansi)
export(output_utf8)
export(utf8_encode)
export(utf8_format)
export(utf8_normalize)
export(utf8_print)
export(utf8_valid)
export(utf8_width)
useDynLib(utf8, .registration = TRUE)
